/*
	PicView v2.1 - displays .PIC Variables in monochrome or grayscale mode
	With Kevin Kofler's 3-Planes-Grayscales-Support since v2.0
    Copyright (C) 2004-2005 Jonas Gehring
        
    PicView is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    PicView is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with PicView; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

// See readme.txt for details & further informations

// C Source File
// Created          28.11.04; 18:31:33
// Updated to v2.0  08.01.05
// Updated to v2.1  20.02.05


#include <alloc.h>					// Routines for dynamic memory allocation
#include <args.h>					// Routines for retrieving arguments passed to the program
#include <asmtypes.h>				// Common type definitions known to ASM programmers
#include <compat.h>					// Various calc-dependent and AMS-dependent pseudo-constants
#include <dialogs.h>				// Routines for creating dialogs
#include <graph.h>					// Common (non-windowed) graphic routines
#include <gray.h>					// Routines for grayscale graphics
#include <intr.h>					// Routines for creating interrupt handlers
#include <kbd.h>					// Keyboard handling routines
#include <statline.h>				// Routines for manipulation of the status line
#include <string.h>					// ANSI-compatible string manipulation routines
#include <system.h>					// System routines (for accessing system timers, queues, etc.)
#include <vat.h>					// Routines for accessing the variable allocation table

#include "gray3P.h"					// GrayScale-Support v3.50-3P for TIGCC by Kevin Kofler			


#define FONT F_4x6
	
enum flags {EXIT, REDRAW_INFO, REDRAW_NO_INFO, SEVEN_SHADES, EIGHT_SHADES};

// Globals:
/*const*/ ESQ types[1] = {PIC_TAG};	// Type for the open-dialog
ESI argptr;	// Pointer to the arguments
HSym Symbl[3];	// Symbol references


// Clears the keyboard queue
inline void ClearQueue()
{
	while (_rowread(0)) continue;	
}


// Dereferences a symbol
SYM_ENTRY *GetEntry(HSym var)
{
	return DerefSym(var);
}


// Waits for a keypress while the picture(s) are displayed
short KeyWait(short flag, short shades)
{
	while (!_keytest(RR_ENTER) && !_keytest(RR_ESC) && !_keytest(RR_ENTER2)) {
		// Toggle displaying of the infos
		if (_keytest(RR_CLEAR)) {
			ClearQueue();
			if (flag == REDRAW_INFO) flag = REDRAW_NO_INFO;
			else flag = REDRAW_INFO;
			return flag;
		}
		
		// Switch between 7 and 8 shades of gray
		if (_keytest(RR_MODE) && shades) {
			ClearQueue();
			if (shades == SEVEN_SHADES) shades = EIGHT_SHADES;
			else shades = EIGHT_SHADES;
			return shades;
		}
	
		// Set Contrast up/down
		if (_keytest(RR_PLUS)) {
			ClearQueue();
			OSContrastUp();
		}
		if (_keytest(RR_MINUS)) {
			ClearQueue();
			OSContrastDn();
		}
	}
	return EXIT;
}


// Routine for Monochrome Picture
void DisplayMonochrome(void)
{
	char fname[23] = {"File:"}, fsize[13], fbuff[17];
	short flag = REDRAW_INFO;

	// Precalculate picture information
	HSYMtoName(Symbl[0], fbuff); 
	strcat(fname, fbuff);
	sprintf(fsize, "Size: %ux%u", ((BITMAP*)(HeapDeref(GetEntry(Symbl[0])->handle)+2))->NumCols, ((BITMAP*)(HeapDeref(GetEntry(Symbl[0])->handle)+2))->NumRows);

	do {
		// Draw the picture onto the screen
		BitmapPut(0, 0, HeapDeref(GetEntry(Symbl[0])->handle)+2, ScrRect, A_NORMAL);
		
		if (flag == REDRAW_INFO) {
			DrawStr(0, LCD_HEIGHT-16, fname,  A_XOR);
			DrawStr(0, LCD_HEIGHT-7, fsize,  A_XOR);
		}	
		
		// Wait for keypress
		if ((flag = KeyWait(flag, 0)) == EXIT)
			return;
		else
			ClrScr();
	} while (TRUE);
}


// Routine for 2-Plane-Grayscale Pictures
void Display2Grayscales(void)
{
	char fname[2][23] = {"File:","File:"}, fsize[2][15], fbuff[17];
	short i;
	short flag = REDRAW_INFO;
	
	// Precalculate picture informations
	for (i = 0; i < 2; i++) {
		HSYMtoName(Symbl[i], fbuff); 
		strcat(fname[i], fbuff);
		sprintf(fsize[i], "Size: %ux%u", ((BITMAP*)(HeapDeref(GetEntry(Symbl[i])->handle)+2))->NumCols, ((BITMAP*)(HeapDeref(GetEntry(Symbl[i])->handle)+2))->NumRows);
		
	}

	do {	
		// Draw the pictures onto the screen
		if (!GrayOn())
			return;

		for (i = 0; i < 2; i++) {
			GraySetAMSPlane(i);		// Force routines to use the given plane
			
			BitmapPut(0, 0, HeapDeref(GetEntry(Symbl[i])->handle)+2, ScrRect,  A_XOR);
							
			if (flag == REDRAW_INFO) {
				DrawStr(0, LCD_HEIGHT-16, fname[0],  A_XOR);
				DrawStr(0, LCD_HEIGHT-7, fsize[0],  A_XOR);
				DrawStr(LCD_WIDTH-DrawStrWidth(fname[1], FONT), LCD_HEIGHT-16, fname[1],  A_XOR);
				DrawStr(LCD_WIDTH-DrawStrWidth(fsize[1], FONT), LCD_HEIGHT-7, fsize[1],  A_XOR);
			}	
		}
		
		// Wait for keypress	
		if ((flag = KeyWait(flag, 0)) == EXIT) {
			GrayOff();
			return;
		} else {
			GrayOff();
			ClrScr();
		}
	} while (TRUE);
}


// Routine for 3-Plane-Grayscale Pictures
void Display3Grayscales(void) 
{
	char fname[3][17], fsize[3][8];
	short i;
	short flag = REDRAW_INFO, shades = SEVEN_SHADES;

	// Precalculate picture informations
	for (i = 0; i < 3; i++) {
		HSYMtoName(Symbl[i], fname[i]); 
		sprintf(fsize[i], "%ux%u", ((BITMAP*)(HeapDeref(GetEntry(Symbl[i])->handle)+2))->NumCols, ((BITMAP*)(HeapDeref(GetEntry(Symbl[i])->handle)+2))->NumRows);		
	}
	
	do {
		// Draw the pictures onto the screen
		if (shades == SEVEN_SHADES) {
			if (!Gray3POn(FALSE))
				return;
		} else {
			if (!Gray3POn(TRUE))
				return;
		}
		
		for (i = 0; i < 3; i++) {
			Gray3PSetAMSPlane(i);	// Force routines to use the given plane
			
			BitmapPut(0, 0, HeapDeref(GetEntry(Symbl[i])->handle)+2, ScrRect, A_NORMAL);
			
			if (flag == REDRAW_INFO) {
				DrawStr(0, LCD_HEIGHT-16, fname[0],  A_XOR);
				DrawStr(0, LCD_HEIGHT-7, fsize[0],  A_XOR);
				DrawStr((LCD_WIDTH/2)-(DrawStrWidth(fname[1], FONT)/2), LCD_HEIGHT-16, fname[1],  A_XOR);
				DrawStr((LCD_WIDTH/2)-(DrawStrWidth(fsize[1], FONT)/2), LCD_HEIGHT-7, fsize[1],  A_XOR);	
				DrawStr(LCD_WIDTH-DrawStrWidth(fname[2], FONT), LCD_HEIGHT-16, fname[2],  A_XOR);
				DrawStr(LCD_WIDTH-DrawStrWidth(fsize[2], FONT), LCD_HEIGHT-7, fsize[2],  A_XOR);				
			}
		}
	
		// Wait for keypress	
		switch (KeyWait(flag, shades)) {
			case EXIT:
				Gray3POff();
				return;
			case REDRAW_INFO: 
				flag = REDRAW_INFO;
				break;
			case REDRAW_NO_INFO:
				flag = REDRAW_NO_INFO;
				break;
			case SEVEN_SHADES:
				shades = SEVEN_SHADES;
				break;
			case EIGHT_SHADES:
				shades = EIGHT_SHADES;
				break;
			default:break;
		}
		
		Gray3POff();
		ClrScr();
	
	} while (TRUE);
}


// Main Function (program entry point)
void _main(void)
{
	LCD_BUFFER *buffer = malloc(LCD_SIZE);
	short mode = 0, mode2;

	INT_HANDLER save_int_1 = GetIntVec(AUTO_INT_1);
	INT_HANDLER save_int_5 = GetIntVec(AUTO_INT_5);
	
	if (!buffer) {
		ST_helpMsg("out of memory");	
		return;
	}
	
	LCD_save(buffer);
	
	// Proof if an argument was passed
	mode2 = ArgCount();
		
	// Getr Pointer to the argument
	InitArgPtr(argptr);
	
	// Is the variable a positive integer?
	if (GetArgType(argptr) != POSINT_TAG)
		mode2 = 0;
	else
		mode2 = 1;
	
	// Proof if an argument was passed
	if (mode2)
		mode = GetIntArg(argptr);

	// If no arguments are passed or if they were wrong
	if (!mode2 || (mode != 0 && mode != 1 && mode != 2)) {
	  HANDLE handle;
  	if (!CALCULATOR)
  		handle = DialogNewSimple(130, 60);
  	else
  		handle = DialogNewSimple(215, 67);
  	DialogAddTitle(handle, "PicView v2.0 - Usage", BT_NONE, BT_OK);
  	DialogAddText(handle, 3, 15, "picview(0): monochrome pics");
  	DialogAddText(handle, 3, 27, "picview(1): 2-plane-grayscale pics");
  	DialogAddText(handle, 3, 39, "picview(2): 3-plane-grayscale pics");
 		DialogDo(handle, CENTER, CENTER, NULL, NULL);
  	HeapFree(handle);
  	return;
	}

	// Open-Dialogs for the pictures
	if (!((Symbl[0] = VarOpen(types)).folder))
		goto quit;
	if (mode) {
		if (!((Symbl[1] = VarOpen(types)).folder))
			goto quit;
	}
	if (mode == 2) {
		if (!((Symbl[2] = VarOpen(types)).folder))
			goto quit;
	}	
	
	ClrScr();

	// Set font for the infos to 'small'			
	FontSetSys(FONT);

	SetIntVec(AUTO_INT_1, DUMMY_HANDLER);
	SetIntVec(AUTO_INT_5, DUMMY_HANDLER);
	
	// Display the pictures
	if (!mode)
		DisplayMonochrome();
	else if (mode == 1)
		Display2Grayscales();
	else if (mode == 2)
		Display3Grayscales();
	
	ClrScr();

	quit:
	// Clear the keyboard queue
	while (_rowread(0)) continue;
	
	// Restore all settins
	SetIntVec(AUTO_INT_1, save_int_1);
	SetIntVec(AUTO_INT_5, save_int_5);
	LCD_restore(buffer);
	HeapFreePtr(buffer);
	
	// Show message
	ST_helpMsg("PicView v2.1 - www.saubue68k.de.vu");
}
